*$ CREATE FLUSCW.FOR
*COPY FLUSCW
*                                                                      *
*=== fluscw ===========================================================*
*                                                                      *
      DOUBLE PRECISION FUNCTION FLUSCW ( IJ    , PLA   , TXX   , TYY   ,
     &                                   TZZ   , WEE   , XX    , YY    ,
     &                                   ZZ    , NREG  , IOLREG, LLO   ,
     &                                   NSURF )

      INCLUDE '(DBLPRC)'
      INCLUDE '(DIMPAR)'
      INCLUDE '(IOUNIT)'
*
*----------------------------------------------------------------------*
*                                                                      *
*     Copyright (C) 1989-2005      by    Alfredo Ferrari & Paola Sala  *
*     All Rights Reserved.                                             *
*                                                                      *
*     New version of Fluscw for FLUKA9x-FLUKA200x:                     *
*                                                                      *
*     !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!     *
*     !!! This is a completely dummy routine for Fluka9x/200x. !!!     *
*     !!! The  name has been kept the same as for older  Fluka !!!     *
*     !!! versions for back-compatibility, even though  Fluscw !!!     *
*     !!! is applied only to estimators which didn't exist be- !!!     *
*     !!! fore Fluka89.                                        !!!     *
*     !!! User  developed versions  can be used for  weighting !!!     *
*     !!! flux-like quantities at runtime                      !!!     *
*     !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!     *
*                                                                      *
*     Input variables:                                                 *
*                                                                      *
*           Ij = (generalized) particle code (Paprop numbering)        *
*          Pla = particle laboratory momentum (GeV/c) (if > 0),        *
*                or kinetic energy (GeV) (if <0 )                      *
*    Txx,yy,zz = particle direction cosines                            *
*          Wee = particle weight                                       *
*     Xx,Yy,Zz = position                                              *
*         Nreg = (new) region number                                   *
*       Iolreg = (old) region number                                   *
*          Llo = particle generation                                   *
*        Nsurf = transport flag (ignore!)                              *
*                                                                      *
*     Output variables:                                                *
*                                                                      *
*       Fluscw = factor the scored amount will be multiplied by        *
*       Lsczer = logical flag, if true no amount will be scored        *
*                regardless of Fluscw                                  *
*                                                                      *
*     Useful variables (common SCOHLP):                                *
*                                                                      *
*     Flux like binnings/estimators (Fluscw):                          *
*          ISCRNG = 1 --> Boundary crossing estimator                  *
*          ISCRNG = 2 --> Track  length     binning                    *
*          ISCRNG = 3 --> Track  length     estimator                  *
*          ISCRNG = 4 --> Collision density estimator                  *
*          ISCRNG = 5 --> Yield             estimator                  *
*          JSCRNG = # of the binning/estimator                         *
*                                                                      *
*----------------------------------------------------------------------*
*

      INCLUDE '(SCOHLP)'
      INCLUDE '(USRBDX)'
      INCLUDE '(PAPROP)'
      INCLUDE '(BEAMCM)'
      INCLUDE 'flukaio.inc'

      INTEGER ILASTID
      SAVE ILASTID
      DATA ILASTID / 0 /

      INTEGER IDP0
      INTEGER IGEN0

*     Send back protons
      IF ( ISCRNG .EQ.1 .AND. TITUSX(JSCRNG) .EQ. "BACK2ICO" ) THEN

*       If generation bigger than 1 generate new particle id
        IF (LLO.GT.1) THEN
          IDP0 = IDP
          IDGEN0 = IDGEN

          IDGEN = IDP

          IF(ILASTID.EQ.IDP) THEN
            IDP = IDP + 10000
          END IF
        ENDIF

* don't overwrite PLA...

        IF ( PLA .GT. 0.0D+00 ) THEN
           PSCO = PLA
        ELSE
*          new formulation (equivalent to the old one),
*          without subtractions! (more precise...)
*          NB: PLA < 0.0 !!
           PSCO = SQRT( -PLA * ( -PLA + TWOTWO * AM(IJ) ) )
        ENDIF

*        WRITE(LUNOUT,*)'END:',IDP, IBARCH(IJ), ICHRGE(IJ)

        N = NTSENDP(
     &            ICID,
     &            IDP, IDGEN, WEE,
     &            XX - XBEAM, YY - YBEAM, ZZ - ZBEAM, 
     &            TXX, TYY, TZZ,
     &            IBARCH(IJ), ICHRGE(IJ),
     &            AM(IJ), PSCO,
     &            T)

        IF (LLO.GT.1) THEN
          IDP = IDP0
          IDGEN = IDGEN0
        ENDIF

        ILASTID = IDP
      ENDIF


      FLUSCW = ONEONE
      LSCZER = .FALSE.

      RETURN

*=== End of function Fluscw ===========================================*
      END


