#ifndef STANDARDCOLLIMATOR_H
#define STANDARDCOLLIMATOR_H
#include <iostream>
#include <vector>
#include <string>
#include <cmath>
#include "Collimator.h"
using namespace std;


/*
#############################################################################################################

This class describes all standard collimators in the accelerator lattice.

#############################################################################################################
*/


class StandardCollimator : public Collimator
{

public:

    //=================================================Constructors, destructor=======================================

    StandardCollimator(const double& ALFX, const double& ALFY, const double& APER_1, const double& APER_2, const double& APER_3, const double& APER_4, const string& APERTYPE, const double& BETX, const double& BETY, const double& DPX, const double& DPY, const double& DX, const double& DY, const string& KEYWORD, const double& L, const double& MUX, const double& MUY, const string& NAME, const double& PTC, const double& PXC, const double& PYC, const double& S, const double& TC, const double& XC, const double& YC, const double& K0L, const double& K0SL, const double& K1L, const double& K1SL, const double& K2L, const double& K2SL, const string& PARENT, const string& meth, const long double& hgap, const long double& hgap2, const double& collang, const long double& pdepth, const long double& pdepth2, const double& tcang, const double& nsig);

    StandardCollimator(Element elt, const double& tcang, const double& nsig, const string& meth, const string& material);

    StandardCollimator(const StandardCollimator& obj);

    virtual ~StandardCollimator() {};


    //describes the passage of a particle through the collimator

    void collipass(Particle& p1, double& dpopeff, const double& scaleorbit, const double& R11X, const double& R12X, const double& R21X, const double& R22X, const double& R11Y, const double& R12Y, const double& R21Y, const double& R22Y, const double& dx1, const double& dpx1, const double& dy1, const double& dpy1, const double& delta_s, const double& Apr, const double& Zpr, const double& betgam);


    //the following methodes are only used in the class FlukaCollimator/CrystalCollimator
#if defined(FLUKA)
    void collipassfluka(vector <Particle>& bunchstart, vector <Particle>& bunchend, flukaio_connection_t* connection, const int& turn, const double& momentum) {};

    void collipassCrystal(vector <Particle>& bunch, const double& betgam, const int& pas, string outputpath) {};
#endif


    //displays the parameters of the collimator

    void affiche();


};

#endif
